require 'sequential_workflow'
RSpec.describe 'SequentialWorkflow' do
  class Foo
    include SequentialWorkflow

    def execute_workflow
      start(:bar)
      .then(:one)
      .then(:two)
      .then(:three, :handle_previous_error)
      .then(:four)
      .catch(:catch_method)
    end

    def bar
    end

    def one(value)
    end

    def two(value)
    end

    def three(value)
    end

    def four(value)
    end

    def handle_previous_error(e)
    end

    def catch_method(e)
    end
  end

  let(:foo) { Foo.new }

  describe '#then' do
    context 'on success' do
      let(:value) { 1 }

      before do
        allow(foo).to receive(:bar).and_return(value)
      end

      it 'sets #workflow_state to :fulfilled' do
        foo.execute_workflow
        expect(foo.workflow_state).to eq :fulfilled
      end

      it 'calls the next method with @value' do
        expect(foo).to receive(:one).with(value)
        foo.execute_workflow
      end
    end

    context 'on rejection' do
      before do
        allow(foo).to receive(:one).and_raise('Error')
      end

      it 'calls the rescue method' do
        expect(foo).to receive(:handle_previous_error).with(RuntimeError)
        foo.execute_workflow
      end

      it 'sets #workflow_state to :rejected' do
        foo.execute_workflow
        expect(foo.workflow_state).to eq :rejected
      end

      it "doesn't call subsequent methods" do
        expect(foo).not_to receive(:two)
        foo.execute_workflow
      end
    end

  end
  describe '#catch' do
    context 'on :rejected' do
      before do
        allow(foo).to receive(:one).and_raise('Error')
      end

      it 'runs the catch method' do
        expect(foo).to receive(:catch_method).with(RuntimeError)
        foo.execute_workflow
      end
    end

    context 'on :fulfilled' do
      it 'does not run the catch method' do
        expect(foo).not_to receive(:catch_method)
        foo.execute_workflow
      end
    end
  end
end
